<?php 

/******************************************************************************
-- HalThumb - a quick and dirty thumbnail script
-- Parameters:
-- src = the path to the image
-- w = width of thumb, in pixels
-- h = height of thumb, in pixels
-- crop = type of crop. Options are portfolio or full
-- http://www.halnesbitt.com
******************************************************************************/

	//determine the path and parts
	$src = $_GET['src'];
	$path_parts = pathinfo($src);
	$ext = $path_parts['extension'];
	$file_name = $path_parts['filename'] . "." . $ext;
	$local_path = $_SERVER['DOCUMENT_ROOT'] . $src;

	//set defaults
	$MAX_WIDTH = 100;
	$MAX_HEIGHT = 100;
	$MAX_WIDTH_LIMIT = 1200;
	$MAX_HEIGHT_LIMIT = 1200;

	//get parameters from querystring
	$MAX_WIDTH = grabParam('w');
	$MAX_HEIGHT = grabParam('h');
	$CROP_TYPE = grabParam('crop');


	//setting limits so people can't just blow up thumbnails
	if ($MAX_WIDTH > $MAX_WIDTH_LIMIT) {
		$MAX_WIDTH = $MAX_WIDTH_LIMIT;
	}
	if ($MAX_HEIGHT > $MAX_HEIGHT_LIMIT) {
		$MAX_HEIGHT = $MAX_HEIGHT_LIMIT;
	}

	//get the photo into data form
	switch ($ext) {
		case "jpg":
			$mime_type = "image/jpeg";
         		$photo = imagecreatefromjpeg($local_path);
         		break;

		case "png":
			$mime_type = "image/png";
         		$photo = imagecreatefrompng($local_path);
         		break;

		case "gif":
			$mime_type = "image/gif";
         		$photo = imagecreatefromgif($local_path);
         		break;
	}

	//getting dimensions of real photo
	$old_width = imagesx($photo);
	$old_height = imagesy($photo);

	//set measurements for the crop and scale
	switch ($CROP_TYPE) {

		case "portfolio":
			//crops and resizes based on passed sizes - 600 x 400
			$thumb_width = $MAX_WIDTH;
			$thumb_height = $MAX_HEIGHT;

			$original_aspect = $old_width / $old_height;
			$thumb_aspect = $thumb_width / $thumb_height;

			if($original_aspect >= $thumb_aspect) {   
				// If image is wider than thumbnail (in aspect ratio sense)   
				$new_height = $thumb_height;   
				$new_width = $old_width / ($old_height / $thumb_height);
			} else {   
				// If the thumbnail is wider than the image   
				$new_width = $thumb_width;   
				$new_height = $old_height / ($old_width / $thumb_width);
			}

			$src_x = ($new_width - $thumb_width) / 2;
			$src_y = ($new_height - $thumb_height) / 2;
			break;
		
		case "full":
			//show the full image at its full, original size
			$new_width = $old_width;
			$new_height = $old_height;
			$thumb_width = $old_width;
			$thumb_height = $old_height;
			$src_x = 0;
			$src_y = 0;
			break;

		default:
			//true scaled
			$scale = min($MAX_WIDTH/$old_width, $MAX_HEIGHT/$old_height);
			$new_width = floor($scale*$old_width);
			$new_height = floor($scale*$old_height);
			$thumb_width = $new_width;
			$thumb_height = $new_height;
			$src_x = 0;
			$src_y = 0;
			break;

	}


	//create the new photo and assign the data
	$tmp_img = imagecreatetruecolor($thumb_width, $thumb_height);

	//handle transparency
	switch ($ext) {
		case "png":
		// integer representation of the color black (rgb: 0,0,0)
		$background = imagecolorallocate($tmp_img, 0, 0, 0);
		// removing the black from the placeholder
		imagecolortransparent($tmp_img, $background);

		// turning off alpha blending (to ensure alpha channel information is preserved, rather than removed (blending with the rest of the image in the form of black))
		imagealphablending($tmp_img, false);

		// turning on alpha channel information saving (to ensure the full range of transparency is preserved)
		imagesavealpha($tmp_img, true);
		break;

	case "gif":
		// integer representation of the color black (rgb: 0,0,0)
		$background = imagecolorallocate($tmp_img, 0, 0, 0);
		// removing the black from the placeholder
		imagecolortransparent($tmp_img, $background);
		break;
	}

	//do it, doug
	imagecopyresampled($tmp_img, $photo, 0, 0, $src_x, $src_y, $new_width, $new_height, $old_width, $old_height);
 	$photo = $tmp_img;        
 
	//set all of the headers
	header("Cache-Control: private, max-age=10800, pre-check=10800");
	header("Pragma: private");
	header("Expires: " . date(DATE_RFC822,strtotime(" 2 day")));

	//commenting out size - was fucking up and making the browser hang, for some reason
	//header("Content-length: $size");
	header("Content-type: $mime_type");
	header("Content-Disposition: inline; filename=$file_name");

	//output based on type
	switch ($ext) {
		case "png":
			imagepng($photo);
         		break;

		case "gif":
			imagegif($photo);
         		break;

		default:
			imagejpeg($photo);
			break;
		
	}

	//clean up
	if(is_resource($photo)) {
		imagedestroy($photo);
	}
	if(is_resource($tmp_img)) {
		imagedestroy($tmp_img);
	}

	exit();


	//misc helper functions
	function grabParam($x) {
		if(isset($_GET[$x]) && $_GET[$x] != "") {
			return $_GET[$x];
		}
	}

?>